@@ -7,6 +7,8 @@ from datetime import datetime
 
 import json
 
+import numpy as np
+
 from PyQt5.QtCore import Qt, QSize
 from PyQt5.QtGui import QFont, QIcon
 from PyQt5.QtWidgets import QWidget, QButtonGroup, QCheckBox, QDialog, QFileDialog, QHBoxLayout, QApplication, QLabel, QMainWindow, QRadioButton, QStackedLayout, QVBoxLayout, QMessageBox, QTextEdit, QSpacerItem, QPushButton
@@ -24,12 +26,15 @@ import zebrazoom.code.GUI.configFileZebrafishFunctions as configFileZebrafishFun
 import zebrazoom.code.GUI.adjustParameterInsideAlgoFunctions as adjustParameterInsideAlgoFunctions
 import zebrazoom.code.GUI.dataAnalysisGUIFunctions as dataAnalysisGUIFunctions
 import zebrazoom.code.GUI.troubleshootingFunction as troubleshootingFunction
-from zebrazoom.mainZZ import mainZZ
+from zebrazoom.zebraZoomVideoAnalysis import ZebraZoomVideoAnalysis
+from zebrazoom.code.adjustHyperparameters import adjustFreelySwimTrackingParams, adjustFreelySwimTrackingAutoParams, adjustHeadEmbededTrackingParams, adjustHeadEmbeddedEyeTrackingParamsSegment, adjustHeadEmbeddedEyeTrackingParamsEllipse
 from zebrazoom.code.GUI.GUI_InitialClasses import StartPage, VideoToAnalyze, ConfigFilePromp, Patience, ZZoutro, ZZoutroSbatch, SeveralVideos, FolderToAnalyze, TailExtremityHE, FolderMultipleROIInitialSelect, EnhanceZZOutput, ViewParameters, Error
 from zebrazoom.code.GUI.configFilePrepare import ChooseVideoToCreateConfigFileFor, OptimizeConfigFile, ChooseGeneralExperiment, ChooseCenterOfMassTracking, WellOrganisation, FreelySwimmingExperiment, NbRegionsOfInterest, HomegeneousWellsLayout, CircularOrRectangularWells, NumberOfAnimals, NumberOfAnimals2, NumberOfAnimalsCenterOfMass, IdentifyHeadCenter, IdentifyBodyExtremity, FinishConfig, ChooseCircularWellsLeft, ChooseCircularWellsRight, GoToAdvanceSettings
 from zebrazoom.code.GUI.configFileZebrafish import HeadEmbeded
 from zebrazoom.code.GUI.dataAnalysisGUI import CreateExperimentOrganizationExcel, ChooseDataAnalysisMethod, PopulationComparison, BoutClustering, AnalysisOutputFolderClustering
+from zebrazoom.code.GUI.rolloverAnalysis import RolloverAnalysis, ManualRolloverClassification
 from zebrazoom.code.GUI.troubleshooting import ChooseVideoToTroubleshootSplitVideo, VideoToTroubleshootSplitVideo
+import zebrazoom.code.GUI.tracking.customTrackingImplementations
 
 
 LARGE_FONT= ("Verdana", 12)
@@ -47,7 +52,7 @@ def excepthook(excType, excValue, traceback_):
   formattedTraceback = traceback.format_exception(excType, excValue, traceback_)
   errorMessage.setText("An error has ocurred: %s" % formattedTraceback[-1])
   informativeText = 'Please report the issue on <a href="https://github.com/oliviermirat/ZebraZoom/issues">Github</a>.'
-  if app.configFile and app.savedConfigFile != {k: v for k, v in app.configFile.items() if k != "firstFrame" and k != "lastFrame"}:
+  if app.configFile and app.savedConfigFile != app.configFile:
     configDir = paths.getConfigurationFolder()
     videoName = os.path.splitext(os.path.basename(app.videoToCreateConfigFileFor))[0]
     configFilename = os.path.join(configDir, '%s_%s_unfinished.json' % (videoName, datetime.now().strftime("%Y_%m_%d-%H_%M_%S")))
@@ -111,7 +116,7 @@ class ZebraZoomApp(PlainApplication):
         self.window.closeEvent = self._windowClosed(self.window, self.window.closeEvent)
         layout = QStackedLayout()
         self.frames = {}
-        for idx, F in enumerate((StartPage, VideoToAnalyze, ConfigFilePromp, Patience, ZZoutro, ZZoutroSbatch, SeveralVideos, FolderToAnalyze, EnhanceZZOutput, TailExtremityHE, FolderMultipleROIInitialSelect, ViewParameters, Error, ChooseVideoToCreateConfigFileFor, OptimizeConfigFile, ChooseGeneralExperiment, ChooseCenterOfMassTracking, WellOrganisation, FreelySwimmingExperiment, NbRegionsOfInterest, HomegeneousWellsLayout, CircularOrRectangularWells, NumberOfAnimals, NumberOfAnimals2, NumberOfAnimalsCenterOfMass, IdentifyHeadCenter, IdentifyBodyExtremity, FinishConfig, ChooseCircularWellsLeft, ChooseCircularWellsRight, GoToAdvanceSettings, HeadEmbeded, ChooseDataAnalysisMethod, PopulationComparison, BoutClustering, AnalysisOutputFolderClustering, ChooseVideoToTroubleshootSplitVideo, VideoToTroubleshootSplitVideo)):
+        for idx, F in enumerate((StartPage, VideoToAnalyze, ConfigFilePromp, Patience, ZZoutro, ZZoutroSbatch, SeveralVideos, FolderToAnalyze, EnhanceZZOutput, TailExtremityHE, FolderMultipleROIInitialSelect, ViewParameters, Error, ChooseVideoToCreateConfigFileFor, OptimizeConfigFile, ChooseGeneralExperiment, ChooseCenterOfMassTracking, WellOrganisation, FreelySwimmingExperiment, NbRegionsOfInterest, HomegeneousWellsLayout, CircularOrRectangularWells, NumberOfAnimals, NumberOfAnimals2, NumberOfAnimalsCenterOfMass, IdentifyHeadCenter, IdentifyBodyExtremity, FinishConfig, ChooseCircularWellsLeft, ChooseCircularWellsRight, GoToAdvanceSettings, HeadEmbeded, ChooseDataAnalysisMethod, PopulationComparison, BoutClustering, AnalysisOutputFolderClustering, ChooseVideoToTroubleshootSplitVideo, VideoToTroubleshootSplitVideo, RolloverAnalysis, ManualRolloverClassification)):
             self.frames[F.__name__] = idx
             page = F(self)
             if hasattr(page, 'preferredSize'):
@@ -136,6 +141,8 @@ class ZebraZoomApp(PlainApplication):
         wrapperLayout = QVBoxLayout()
         wrapperLayout.addWidget(page, alignment=Qt.AlignmentFlag.AlignCenter)
         wrapperWidget.setLayout(wrapperLayout)
+        if hasattr(page, 'setArgs'):
+          wrapperWidget.setArgs = page.setArgs
         return wrapperWidget
 
     def _currentPageChanged(self):
@@ -150,7 +157,7 @@ class ZebraZoomApp(PlainApplication):
     def show_frame(self, page_name):
         '''Show a frame for the given page name'''
         if page_name == "StartPage":
-            if self.configFile and self.savedConfigFile != {k: v for k, v in self.configFile.items() if k != "firstFrame" and k != "lastFrame"}:
+            if self.configFile and self.savedConfigFile != self.configFile:
                 reply = QMessageBox.question(self.window, "Unsaved Changes",
                                              "Are you sure you want to go back to the start page? Changes made to the config will be lost.",
                                              QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No)
@@ -176,9 +183,6 @@ class ZebraZoomApp(PlainApplication):
           layout.currentChanged.connect(cleanup)
         else:
           self.window.centralWidget().layout().setCurrentIndex(self.frames[page_name])
-        if page_name == "FinishConfig":
-          fps = zzVideoReading.VideoCapture(self.videoToCreateConfigFileFor).get(5)
-          self.window.centralWidget().layout().currentWidget().setFPS(fps)
 
     def _windowClosed(self, window, fn):
         def inner(*args, **kwargs):
@@ -212,8 +216,8 @@ class ZebraZoomApp(PlainApplication):
         if self._busyCursor:
             self.setOverrideCursor(Qt.CursorShape.BusyCursor)
 
-    def chooseVideoToAnalyze(self, justExtractParams, noValidationVideo, chooseFrames, testMode):
-        GUI_InitialFunctions.chooseVideoToAnalyze(self, justExtractParams, noValidationVideo, chooseFrames, testMode)
+    def chooseVideoToAnalyze(self, justExtractParams, noValidationVideo, chooseFrames):
+        GUI_InitialFunctions.chooseVideoToAnalyze(self, justExtractParams, noValidationVideo, chooseFrames)
 
     def chooseFolderToAnalyze(self, justExtractParams, noValidationVideo, sbatchMode):
         GUI_InitialFunctions.chooseFolderToAnalyze(self, justExtractParams, noValidationVideo, sbatchMode)
@@ -224,16 +228,14 @@ class ZebraZoomApp(PlainApplication):
     def chooseFolderForMultipleROIs(self, askCoordinatesForAll):
         GUI_InitialFunctions.chooseFolderForMultipleROIs(self, askCoordinatesForAll)
 
-    def chooseConfigFile(self):
-        GUI_InitialFunctions.chooseConfigFile(self)
-
-    def launchZebraZoom(self):
-        GUI_InitialFunctions.launchZebraZoom(self)
-
     def showViewParameters(self, folder=None):
         self.show_frame("ViewParameters")
         self.window.centralWidget().layout().currentWidget().setFolder(folder)
 
+    def showManualRolloverClassification(self, folder=None):
+        self.show_frame("ManualRolloverClassification")
+        self.window.centralWidget().layout().currentWidget().setFolder(folder)
+
     def openConfigurationFileFolder(self, homeDirectory):
         GUI_InitialFunctions.openConfigurationFileFolder(self, homeDirectory)
 
@@ -278,9 +280,11 @@ class ZebraZoomApp(PlainApplication):
           return
         # Ideally would like to remove these four lines below, once the problem with wrong 'firstFrame' and 'lastFrame' being saved in the configuration file is solved
         if "lastFrame" in self.configFile:
-          del self.configFile["lastFrame"]
+          # del self.configFile["lastFrame"]
+          self.configFile["lastFrameForBackExtract"] = self.configFile["lastFrame"]
         if "firstFrame" in self.configFile:
-          del self.configFile["firstFrame"]
+          # del self.configFile["firstFrame"]
+          self.configFile["firstFrameForBackExtract"] = self.configFile["firstFrame"]
 
         with open(reference, 'w') as outfile:
           json.dump(self.configFile, outfile)
@@ -322,12 +326,12 @@ class ZebraZoomApp(PlainApplication):
     def calculateBackground(self, controller, nbImagesForBackgroundCalculation, useNext=True):
       adjustParameterInsideAlgoFunctions.calculateBackground(self, controller, nbImagesForBackgroundCalculation, useNext)
 
-    def calculateBackgroundFreelySwim(self, controller, nbImagesForBackgroundCalculation, morePreciseFastScreen=False, automaticParameters=False, boutDetectionsOnly=False, useNext=True, nextCb=None):
-      adjustParameterInsideAlgoFunctions.calculateBackgroundFreelySwim(self, controller, nbImagesForBackgroundCalculation, morePreciseFastScreen, automaticParameters, boutDetectionsOnly, useNext, nextCb)
+    def calculateBackgroundFreelySwim(self, controller, nbImagesForBackgroundCalculation, morePreciseFastScreen=False, automaticParameters=False, boutDetectionsOnly=False, useNext=True, nextCb=None, reloadWellPositions=False):
+      adjustParameterInsideAlgoFunctions.calculateBackgroundFreelySwim(self, controller, nbImagesForBackgroundCalculation, morePreciseFastScreen, automaticParameters, boutDetectionsOnly, useNext, nextCb, reloadWellPositions=reloadWellPositions)
 
     def goToAdvanceSettings(self, controller, yes, no):
       configFilePrepareFunctions.goToAdvanceSettings(self, controller, yes, no)
-      
+
     def boutClustering(self, controller, nbClustersToFind, FreelySwimming, HeadEmbeded, minNbBendForBoutDetect=3, nbVideosToSave=0, modelUsedForClustering='', removeOutliers=False, frameStepForDistanceCalculation='4', removeBoutsContainingNanValuesInParametersUsedForClustering=True, forcePandasRecreation=0):
       dataAnalysisGUIFunctions.boutClustering(self, controller, nbClustersToFind, FreelySwimming, HeadEmbeded, minNbBendForBoutDetect, nbVideosToSave, modelUsedForClustering, removeOutliers, frameStepForDistanceCalculation, removeBoutsContainingNanValuesInParametersUsedForClustering, forcePandasRecreation)
 
@@ -354,11 +358,11 @@ class ZebraZoomApp(PlainApplication):
         del self.configFileHistory[:]
         with self.busyCursor():
           try:
-            tabParams = ["mainZZ", pathToVideo, videoName, videoExt, self.configFile,
+            tabParams = ["zebraZoomVideoAnalysis", pathToVideo, videoName, videoExt, self.configFile,
                          "firstFrame", firstFrame, "lastFrame", lastFrame, "freqAlgoPosFollow", 100,
                          "popUpAlgoFollow", 1, "outputFolder", self.ZZoutputLocation,
                          "backgroundExtractionForceUseAllVideoFrames", int(backgroundExtractionForceUseAllVideoFramesCheckbox.isChecked())]
-            mainZZ(pathToVideo, videoName, videoExt, self.configFile, tabParams)
+            ZebraZoomVideoAnalysis(pathToVideo, videoName, videoExt, self.configFile, tabParams).run()
           except NameError:
             self.show_frame("Error")
             self.ZZoutputLocation = outputLocation
