@@ -4,24 +4,104 @@ import cv2
 import zebrazoom.videoFormatConversion.zzVideoReading as zzVideoReading
 import math
 from zebrazoom.code.getHyperparameters import getHyperparametersSimple
-from zebrazoom.code.getImage.getForegroundImage import getForegroundImage
-from zebrazoom.mainZZ import mainZZ
+from zebrazoom.zebraZoomVideoAnalysis import ZebraZoomVideoAnalysis
 import pickle
 import json
 import os
 import re
-from zebrazoom.code.getBackground import getBackground
 from zebrazoom.code.findWells import findWells
-from zebrazoom.code.trackingFolder.tracking import tracking
 from zebrazoom.code.GUI.adjustParameterInsideAlgoFunctions import prepareConfigFileForParamsAdjustements
-from zebrazoom.code.resizeImageTooLarge import resizeImageTooLarge
 import zebrazoom.code.paths as paths
 import zebrazoom.code.util as util
 
-from PyQt5.QtWidgets import QApplication
+from PyQt5.QtCore import Qt
+from PyQt5.QtWidgets import QApplication, QHBoxLayout, QLabel, QPushButton, QVBoxLayout
+
+from zebrazoom.code.tracking import get_default_tracking_method
 
 
-def getGroundTruthFromUser(self, controller, nbOfImagesToManuallyClassify, saveIntermediary, zebrafishToTrack):
+def _selectWell(app, cap, wellPositions):
+  layout = QVBoxLayout()
+
+  app.wellPositions = [(position['topLeftX'], position['topLeftY'], position['lengthX'], position['lengthY'])
+                       for position in wellPositions]
+
+  firstFrame = app.configFile.get("firstFrame", 1)
+  maxFrame = app.configFile.get("lastFrame", cap.get(7) - 1)
+  frameSlider = util.SliderWithSpinbox(firstFrame, 0, maxFrame, name="Frame")
+
+  def getFrame():
+    cap.set(1, frameSlider.value())
+    ret, frame = cap.read()
+    return frame
+  frameSlider.valueChanged.connect(lambda: util.setPixmapFromCv(getFrame(), video))
+
+  img = getFrame()
+  height, width = img.shape[:2]
+  video = util.WellSelectionLabel(width, height)
+  layout.addWidget(video, alignment=Qt.AlignmentFlag.AlignCenter, stretch=1)
+
+  sublayout = QHBoxLayout()
+  sublayout.addStretch(1)
+  sublayout.addWidget(frameSlider, alignment=Qt.AlignmentFlag.AlignCenter)
+  if maxFrame - firstFrame > 1000:
+    adjustLayout = QVBoxLayout()
+    adjustLayout.setSpacing(0)
+    adjustLayout.addStretch()
+    zoomInSliderBtn = QPushButton("Zoom in slider")
+
+    def updatePreciseFrameSlider(value):
+      if frameSlider.minimum() == value and frameSlider.minimum():
+        frameSlider.setMinimum(frameSlider.minimum() - 1)
+        frameSlider.setMaximum(frameSlider.maximum() - 1)
+      elif value == frameSlider.maximum() and frameSlider.maximum() != maxFrame:
+        frameSlider.setMinimum(frameSlider.minimum() + 1)
+        frameSlider.setMaximum(frameSlider.maximum() + 1)
+
+    def zoomInButtonClicked():
+      if "in" in zoomInSliderBtn.text():
+        zoomInSliderBtn.setText("Zoom out slider")
+        value = frameSlider.value()
+        minimum = value - 250
+        maximum = value + 250
+        if minimum < 0:
+          maximum = 500
+          minimum = 0
+        if maximum > frameSlider.maximum():
+          maximum = frameSlider.maximum()
+          minimum = maximum - 500
+        frameSlider.setMinimum(max(0, minimum))
+        frameSlider.setMaximum(min(frameSlider.maximum(), maximum))
+        frameSlider.setValue(value)
+        frameSlider.valueChanged.connect(updatePreciseFrameSlider)
+      else:
+        zoomInSliderBtn.setText("Zoom in slider")
+        frameSlider.setMinimum(0)
+        frameSlider.setMaximum(maxFrame)
+        frameSlider.valueChanged.disconnect(updatePreciseFrameSlider)
+    zoomInSliderBtn.clicked.connect(zoomInButtonClicked)
+    adjustLayout.addWidget(QLabel())
+    adjustLayout.addWidget(zoomInSliderBtn, alignment=Qt.AlignmentFlag.AlignLeft, stretch=1)
+    adjustLayout.addStretch()
+    sublayout.addLayout(adjustLayout)
+  sublayout.addStretch(1)
+  layout.addLayout(sublayout)
+
+  cancelled = False
+  def cancelCb():
+    nonlocal cancelled
+    cancelled = True
+  buttons = (("Cancel", cancelCb, True), ("Ok", None, True, None, util.DEFAULT_BUTTON_COLOR))
+  util.showBlockingPage(layout, title='Select the well with the most movement', buttons=buttons, labelInfo=(img, video, False))
+  wellNumber = None if cancelled else video.getWell()
+  del app.wellPositions
+  return wellNumber
+
+
+def getGroundTruthFromUser(self, controller, nbOfImagesToManuallyClassify, saveIntermediary, zebrafishToTrack, wellNumber=None):
+  
+  firstFrameIndex = self.configFile["firstFrame"] if "firstFrame" in self.configFile else -1
+  lastFrameIndex  = self.configFile["lastFrame"]  if "lastFrame"  in self.configFile else -1
   
   videoPath = self.videoToCreateConfigFileFor
   
@@ -38,98 +118,148 @@ def getGroundTruthFromUser(self, controller, nbOfImagesToManuallyClassify, saveI
   
   initialHyperparameters = getHyperparametersSimple(initialConfigFile)
   initialHyperparameters["videoName"]      = videoName
-  wellPositions = findWells(os.path.join(pathToVideo, videoNameWithExt), initialHyperparameters)
-  if wellPositions is None:
+  initialConfigFile["exitAfterWellsDetection"] = 1
+  app = QApplication.instance()
+  app.wellPositions = wellPositions = []
+  try:
+    with app.busyCursor():
+      ZebraZoomVideoAnalysis(pathToVideo, videoName, videoExt, initialConfigFile, []).run()
+  except ValueError:
+    pass
+  finally:
+    del initialConfigFile["exitAfterWellsDetection"]
+    del app.wellPositions
+  if not wellPositions:
     return None
+  wellPositions = [dict(zip(('topLeftX', 'topLeftY', 'lengthX', 'lengthY'), values)) for values in wellPositions]
 
   cap   = zzVideoReading.VideoCapture(videoPath)
   max_l = int(cap.get(7))
-  
-  # Finding the well with the most movement
-  cap.set(1, 0)
-  ret, firstFrame = cap.read()
-  firstFrame = cv2.cvtColor(firstFrame, cv2.COLOR_BGR2GRAY)
-  cap.set(1, max_l-1)
-  ret, lastFrame = cap.read()
-  while not(ret):
-    max_l = max_l - 1
+
+  if wellNumber is None:
+    # Finding the well with the most movement
+    cap.set(1, 0)
+    ret, firstFrame = cap.read()
+    firstFrame = cv2.cvtColor(firstFrame, cv2.COLOR_BGR2GRAY)
     cap.set(1, max_l-1)
     ret, lastFrame = cap.read()
-  lastFrame = cv2.cvtColor(lastFrame, cv2.COLOR_BGR2GRAY)
-  pixelsChange = np.zeros((len(wellPositions)))
-  for wellNumberId in range(0, len(wellPositions)):
-    xtop  = wellPositions[wellNumberId]['topLeftX']
-    ytop  = wellPositions[wellNumberId]['topLeftY']
-    lenX  = wellPositions[wellNumberId]['lengthX']
-    lenY  = wellPositions[wellNumberId]['lengthY']
-    firstFrameROI = firstFrame[ytop:ytop+lenY, xtop:xtop+lenX]
-    lastFrameROI  = lastFrame[ytop:ytop+lenY, xtop:xtop+lenX]
-    pixelsChange[wellNumberId] = np.sum(abs(firstFrameROI-lastFrameROI)) / np.sum(abs(firstFrameROI)) if np.sum(abs(firstFrameROI)) else 0
-  wellNumber = np.argmax(pixelsChange)
+    while not(ret):
+      max_l = max_l - 1
+      cap.set(1, max_l-1)
+      ret, lastFrame = cap.read()
+    lastFrame = cv2.cvtColor(lastFrame, cv2.COLOR_BGR2GRAY)
+    pixelsChange = np.zeros((len(wellPositions)))
+    for wellNumberId in range(0, len(wellPositions)):
+      xtop  = wellPositions[wellNumberId]['topLeftX']
+      ytop  = wellPositions[wellNumberId]['topLeftY']
+      lenX  = wellPositions[wellNumberId]['lengthX']
+      lenY  = wellPositions[wellNumberId]['lengthY']
+      firstFrameROI = firstFrame[ytop:ytop+lenY, xtop:xtop+lenX]
+      lastFrameROI  = lastFrame[ytop:ytop+lenY, xtop:xtop+lenX]
+      pixelsChange[wellNumberId] = np.sum(abs(firstFrameROI-lastFrameROI)) / np.sum(abs(firstFrameROI)) if np.sum(abs(firstFrameROI)) else 0
+    wellNumber = np.argmax(pixelsChange)
   
-  backCalculationStep = int(max_l / nbOfImagesToManuallyClassify)
+  backCalculationStep = int(max_l / nbOfImagesToManuallyClassify) if (firstFrameIndex == -1 or lastFrameIndex == -1) else int((lastFrameIndex - firstFrameIndex + 1) / nbOfImagesToManuallyClassify)
   data = [None] * ((max_l - 1) // backCalculationStep + 1)
   
-  k = 0
-  while k < max_l:
+  k = firstFrameIndex if firstFrameIndex != -1 else 0
+  firstK = k
+  while k < (lastFrameIndex if lastFrameIndex != -1 else max_l):
       
     cap.set(1, k)
     ret, frame = cap.read()
+    
+    if ret:
+      
+      xtop  = wellPositions[wellNumber]['topLeftX']
+      ytop  = wellPositions[wellNumber]['topLeftY']
+      lenX  = wellPositions[wellNumber]['lengthX']
+      lenY  = wellPositions[wellNumber]['lengthY']
+      frame = frame[ytop:ytop+lenY, xtop:xtop+lenX]
 
-    xtop  = wellPositions[wellNumber]['topLeftX']
-    ytop  = wellPositions[wellNumber]['topLeftY']
-    lenX  = wellPositions[wellNumber]['lengthX']
-    lenY  = wellPositions[wellNumber]['lengthY']
-    frame = frame[ytop:ytop+lenY, xtop:xtop+lenX]
+      changeWell = False
+      def changeWellCallback(video):
+        nonlocal changeWell
+        changeWell = True
 
-    def callback():
-      nonlocal k
-      k -= backCalculationStep
-    oldk = k
-    if data[k//backCalculationStep] is None:
-      headCoordinates = list(util.getPoint(frame, "Click on the center of the head of one animal" if zebrafishToTrack else "Click on the center of mass of an animal",
-                                           backBtnCb=callback, zoomable=True))
-    else:
-      headCoordinates = data[k//backCalculationStep]["headCoordinates"]
-    if oldk != k:
-      if k >= 0:
-        continue
-      elif initialHyperparameters["groupOfMultipleSameSizeAndShapeEquallySpacedWells"] or initialHyperparameters["multipleROIsDefinedDuringExecution"]:
-        return getGroundTruthFromUser(self, controller, nbOfImagesToManuallyClassify, saveIntermediary, zebrafishToTrack)
+      def callback():
+        nonlocal k
+        k -= backCalculationStep
+      oldk = k
+      if data[k//backCalculationStep] is None:
+        extraButtons = (('Switch well', changeWellCallback, True),)
+        headCoordinates = list(util.getPoint(frame, "Click on the center of the head of one animal" if zebrafishToTrack else "Click on the center of mass of an animal",
+                                             backBtnCb=callback, zoomable=True, extraButtons=extraButtons))
+        while changeWell:
+          newWellNumber = _selectWell(controller, cap, wellPositions)
+          if newWellNumber is None:
+            changeWell = False
+            headCoordinates = list(util.getPoint(frame, "Click on the center of the head of one animal" if zebrafishToTrack else "Click on the center of mass of an animal",
+                                                 backBtnCb=callback, zoomable=True, extraButtons=extraButtons))
+          else:
+            return getGroundTruthFromUser(self, controller, nbOfImagesToManuallyClassify, saveIntermediary, zebrafishToTrack, wellNumber=newWellNumber)
       else:
-        QApplication.instance().configFileHistory[-2]()
+        headCoordinates = data[k//backCalculationStep]["headCoordinates"]
+      if oldk != k:
+        if k >= 0:
+          continue
+        elif initialHyperparameters["groupOfMultipleSameSizeAndShapeEquallySpacedWells"] or initialHyperparameters["multipleROIsDefinedDuringExecution"]:
+          return getGroundTruthFromUser(self, controller, nbOfImagesToManuallyClassify, saveIntermediary, zebrafishToTrack, wellNumber=wellNumber)
+        else:
+          QApplication.instance().configFileHistory[-2]()
+          return None
+      frame2 = cv2.circle(frame, tuple(headCoordinates), 2, (0, 0, 255), -1)
+
+      goToOptimize = False
+      def callback2():
+        nonlocal goToOptimize
+        goToOptimize = True
+      extraButtons = (('I want to manually adjust tracking parameters', callback2, True, None),) if k == firstK else ()
+      extraButtons += (('Switch well', changeWellCallback, True),)
+      tailTipCoordinates = list(util.getPoint(frame2, "Click on the tip of the tail of the same animal" if zebrafishToTrack else "Click on a point on the border of the same animal",
+                                backBtnCb=callback, zoomable=True, extraButtons=extraButtons))
+      if goToOptimize:
+        # TODO: add calculations here
+        util.addToHistory(controller.optimizeConfigFile)()
         return None
-    frame2 = cv2.circle(frame, tuple(headCoordinates), 2, (0, 0, 255), -1)
-    tailTipCoordinates = list(util.getPoint(frame2, "Click on the tip of the tail of the same animal" if zebrafishToTrack else "Click on a point on the border of the same animal",
-                              backBtnCb=callback, zoomable=True))
-    if oldk != k:
-      k = oldk
-      if data[k//backCalculationStep] is not None:
-        data[k//backCalculationStep] = None
-      continue
 
-    if True: # Centered on the animal
-      minX = min(headCoordinates[0], tailTipCoordinates[0])
-      maxX = max(headCoordinates[0], tailTipCoordinates[0])
-      minY = min(headCoordinates[1], tailTipCoordinates[1])
-      maxY = max(headCoordinates[1], tailTipCoordinates[1])
-      lengthX = maxX - minX
-      lengthY = maxY - minY
-      
-      widdeningFactor = 2
-      minX = minX - int(widdeningFactor * lengthX)
-      maxX = maxX + int(widdeningFactor * lengthX)
-      minY = minY - int(widdeningFactor * lengthY)
-      maxY = maxY + int(widdeningFactor * lengthY)
-      
-      oneWellManuallyChosenTopLeft     = [minX, minY]
-      oneWellManuallyChosenBottomRight = [maxX, maxY]
-    else: # Focused on the initial well
-      oneWellManuallyChosenTopLeft     = [xtop, ytop]
-      oneWellManuallyChosenBottomRight = [xtop + lenX, ytop + lenY]
+      while changeWell:
+        newWellNumber = _selectWell(controller, cap, wellPositions)
+        if newWellNumber is None:
+          changeWell = False
+          tailTipCoordinates = list(util.getPoint(frame2, "Click on the tip of the tail of the same animal" if zebrafishToTrack else "Click on a point on the border of the same animal",
+                                                  backBtnCb=callback, zoomable=True, extraButtons=extraButtons))
+        else:
+          return getGroundTruthFromUser(self, controller, nbOfImagesToManuallyClassify, saveIntermediary, zebrafishToTrack, wellNumber=newWellNumber)
 
-    data[k//backCalculationStep] = {"image": frame, "headCoordinates": headCoordinates, "tailTipCoordinates": tailTipCoordinates, "oneWellManuallyChosenTopLeft": oneWellManuallyChosenTopLeft, "oneWellManuallyChosenBottomRight": oneWellManuallyChosenBottomRight, "frameNumber": k, "wellNumber": wellNumber}
-    k += backCalculationStep
+      if oldk != k:
+        k = oldk
+        if data[k//backCalculationStep] is not None:
+          data[k//backCalculationStep] = None
+        continue
+
+      if True: # Centered on the animal
+        minX = min(headCoordinates[0], tailTipCoordinates[0])
+        maxX = max(headCoordinates[0], tailTipCoordinates[0])
+        minY = min(headCoordinates[1], tailTipCoordinates[1])
+        maxY = max(headCoordinates[1], tailTipCoordinates[1])
+        lengthX = maxX - minX
+        lengthY = maxY - minY
+        
+        widdeningFactor = 2
+        minX = minX - int(widdeningFactor * lengthX)
+        maxX = maxX + int(widdeningFactor * lengthX)
+        minY = minY - int(widdeningFactor * lengthY)
+        maxY = maxY + int(widdeningFactor * lengthY)
+        
+        oneWellManuallyChosenTopLeft     = [minX, minY]
+        oneWellManuallyChosenBottomRight = [maxX, maxY]
+      else: # Focused on the initial well
+        oneWellManuallyChosenTopLeft     = [xtop, ytop]
+        oneWellManuallyChosenBottomRight = [xtop + lenX, ytop + lenY]
+
+      data[k//backCalculationStep] = {"image": frame, "headCoordinates": headCoordinates, "tailTipCoordinates": tailTipCoordinates, "oneWellManuallyChosenTopLeft": oneWellManuallyChosenTopLeft, "oneWellManuallyChosenBottomRight": oneWellManuallyChosenBottomRight, "frameNumber": k, "wellNumber": wellNumber}
+      k += backCalculationStep
 
   if saveIntermediary:
     toSave    = [initialConfigFile, videoPath, data, wellPositions, pathToVideo, videoNameWithExt, videoName, videoExt, zebrafishToTrack]
@@ -141,8 +271,7 @@ def getGroundTruthFromUser(self, controller, nbOfImagesToManuallyClassify, saveI
 
 
 def evaluateMinPixelDiffForBackExtractForCenterOfMassTracking(videoPath, background, image, wellPositions, hyperparameters, tailTipGroundTruth):
-  
-  [foregroundImage, o1, o2] = getForegroundImage(videoPath, background, image["frameNumber"], image["wellNumber"], wellPositions, hyperparameters)
+  [foregroundImage, o1, o2] = get_default_tracking_method()(videoPath, wellPositions, hyperparameters).getForegroundImage(background, image["frameNumber"], image["wellNumber"])
   ret, thresh = cv2.threshold(foregroundImage, hyperparameters["thresholdForBlobImg"], 255, cv2.THRESH_BINARY)
   thresh[0,:] = 255
   thresh[len(thresh)-1,:] = 255
@@ -163,6 +292,8 @@ def evaluateMinPixelDiffForBackExtractForCenterOfMassTracking(videoPath, backgro
 
 def findBestBackgroundSubstractionParameterForEachImage(data, videoPath, background, wellPositions, hyperparameters, videoName, zebrafishToTrack):
   
+  data = [i for i in data if i]
+  
   for image in data:
     
     oneWellManuallyChosenTopLeft     = image["oneWellManuallyChosenTopLeft"]
@@ -187,7 +318,7 @@ def findBestBackgroundSubstractionParameterForEachImage(data, videoPath, backgro
       hyperparameters["minPixelDiffForBackExtract"] = minPixelDiffForBackExtract
       tailTipGroundTruth = image["tailTipCoordinates"]
       if zebrafishToTrack:
-        trackingData = tracking(videoPath, background, image["wellNumber"], wellPositions, hyperparameters, videoName)
+        trackingData = get_default_tracking_method()(videoPath, wellPositions, hyperparameters).runTracking(image["wellNumber"], background=background)
         tailTipPredicted = trackingData[0][0][0][len(trackingData[0][0][0])-1]
         if (trackingData[0][0][0][0][0] == tailTipPredicted[0] and trackingData[0][0][0][0][1] == tailTipPredicted[1]) or (trackingData[0][0][0][1][0] == tailTipPredicted[0] and trackingData[0][0][0][1][1] == tailTipPredicted[1]):
           tailTipDistError = 1000000000
@@ -217,7 +348,7 @@ def findBestBackgroundSubstractionParameterForEachImage(data, videoPath, backgro
     
     if lowestTailTipDistError != 1000000000:
       hyperparameters["minPixelDiffForBackExtract"] = image["bestMinPixelDiffForBackExtract"]
-      [foregroundImage, o1, o2] = getForegroundImage(videoPath, background, image["frameNumber"], image["wellNumber"], wellPositions, hyperparameters)
+      [foregroundImage, o1, o2] = get_default_tracking_method()(videoPath, wellPositions, hyperparameters).getForegroundImage(background, image["frameNumber"], image["wellNumber"])
       ret, thresh = cv2.threshold(foregroundImage, hyperparameters["thresholdForBlobImg"], 255, cv2.THRESH_BINARY)
       thresh[0,:] = 255
       thresh[len(thresh)-1,:] = 255
@@ -250,7 +381,7 @@ def findInitialBlobArea(data, videoPath, background, wellPositions, hyperparamet
       
       bodyContourArea = image["bodyContourArea"]
       
-      [foregroundImage, o1, o2] = getForegroundImage(videoPath, background, image["frameNumber"], image["wellNumber"], wellPositions, hyperparameters)
+      [foregroundImage, o1, o2] = get_default_tracking_method()(videoPath, wellPositions, hyperparameters).getForegroundImage(background, image["frameNumber"], image["wellNumber"])
       
       ret, thresh = cv2.threshold(foregroundImage, hyperparameters["thresholdForBlobImg"], 255, cv2.THRESH_BINARY)
       thresh[0,:] = 255
@@ -279,17 +410,19 @@ def boutDetectionParameters(data, configFile, pathToVideo, videoName, videoExt,
   configFile["exitAfterBackgroundExtraction"] = 1
   configFile["debugExtractBack"]              = 1
   configFile["debugFindWells"]                = 1
+  configFile["reloadWellPositions"] = 1
   
   app = QApplication.instance()
   with app.busyCursor():
     try:
-      mainZZ(pathToVideo, videoName, videoExt, configFile, [])
+      ZebraZoomVideoAnalysis(pathToVideo, videoName, videoExt, configFile, []).run()
     except ValueError:
       configFile["exitAfterBackgroundExtraction"] = 0
   
   del configFile["exitAfterBackgroundExtraction"]
   del configFile["debugExtractBack"]
   del configFile["debugFindWells"]
+  del configFile["reloadWellPositions"]
 
   # Finding the frame with the most movement
   
@@ -366,7 +499,7 @@ def boutDetectionParameters(data, configFile, pathToVideo, videoName, videoExt,
   
   with app.busyCursor():
     try:
-      mainZZ(pathToVideo, videoName, videoExt, configFile, [])
+      ZebraZoomVideoAnalysis(pathToVideo, videoName, videoExt, configFile, []).run()
     except ValueError:
       newhyperparameters = pickle.load(open(os.path.join(paths.getRootDataFolder(), 'newhyperparameters'), 'rb'))
       for index in newhyperparameters:
