@@ -1,8 +1,6 @@
 import h5py
 import numpy as np
-import cv2
 import math
-import json
 import os
 import sys
 import pandas as pd
@@ -13,9 +11,7 @@ import numpy as np
 # from filterpy.common import Q_discrete_white_noise
 
 from zebrazoom.code.detectMovementWithRawVideo import detectMovementWithRawVideo
-from zebrazoom.code.trackingFolder.tailTrackingFunctionsFolder.getTailTipManual import getHeadPositionByFileSaved, getTailTipByFileSaved
 
-import pdb
 
 def distBetweenThetas(theta1, theta2):
   diff = 0
@@ -86,6 +82,9 @@ def smoothAllTailAngles(allAngles, hyperparameters, start, end):
     tailangles_arr_smoothed = np.append(tailangles_arr_smoothed, tailSmoothed2, axis=0)
   return [tailangles_arr, tailangles_arr_smoothed]
 
+def _calculateTailLength(tailX, tailY):
+  return np.sum([math.sqrt((tailX[x] - tailX[x+1]) ** 2 + (tailY[x] - tailY[x+1]) ** 2) for x in range(len(tailX) - 1)])
+
 def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, wellPositions, background, tailAngle = 0):
 
   firstFrame = hyperparameters["firstFrame"]
@@ -184,16 +183,15 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
       tailX[i]   = trackingTail[i,:,0]
       tailY[i]   = trackingTail[i,:,1]
     
-    if hyperparameters["saveAllDataEvenIfNotInBouts"]:
-      trackingFlatten = [trackingHeadTailAllAnimals[animalId][i].flatten().tolist() + [trackingHeadingAllAnimals[animalId][i]] + angle[i].tolist() for i in range(0, len(trackingHeadTailAllAnimals[animalId]))]
+    if hyperparameters["saveAllDataEvenIfNotInBouts"] or hyperparameters["storeH5"]:
+      trackingFlatten = [trackingHeadTailAllAnimals[animalId][i].flatten().tolist() + [_calculateTailLength(tailX[i], tailY[i])] + heading[i].tolist() + angle[i].tolist() for i in range(0, len(heading))]
       trackingFlattenColumnsNames = ['HeadPosX', 'HeadPosY']
-      for i in range(0, int((len(trackingHeadTailAllAnimals[0][0].flatten().tolist()) - 2) / 2)):
+      for i in range(0, (len(trackingHeadTailAllAnimals[0][0].flatten().tolist()) - 2) // 2):
         trackingFlattenColumnsNames += ['TailPosX' + str(i + 1)]
         trackingFlattenColumnsNames += ['TailPosY' + str(i + 1)]
+      trackingFlattenColumnsNames.append('TailLength')
       trackingFlattenColumnsNames += ['Heading']
       trackingFlattenColumnsNames += ['tailAngle']
-      trackingFlattenPandas = pd.DataFrame(np.array(trackingFlatten), columns=trackingFlattenColumnsNames)
-      trackingFlattenPandas.to_csv(os.path.join(os.path.join(hyperparameters["outputFolder"], hyperparameters["videoName"]), 'allData_' + hyperparameters["videoName"] + '_wellNumber' + str(wellNumber) + '_animal' + str(animalId) + '.cvs'))
 
     
     if hyperparameters["noBoutsDetection"] == 1:
@@ -321,7 +319,12 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
       if hyperparameters["addOneFrameAtTheEndForBoutDetection"]:
         if bouts[numBout][2] + 1 < nbFrames:
           bouts[numBout][2] = bouts[numBout][2] + 1
-    
+
+    if hyperparameters["saveAllDataEvenIfNotInBouts"] and not hyperparameters['noBoutsDetection']:
+      for frameData in trackingFlatten:
+        frameData.append(float('nan'))
+      trackingFlattenColumnsNames.append('BoutNumber')
+
     for i in range(0,len(bouts)):
     
       if (hyperparameters["freqAlgoPosFollow"] != 0) and (i % (hyperparameters["freqAlgoPosFollow"]*10) == 0):
@@ -334,7 +337,11 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
       item["BoutStart"]     = start + firstFrame
       item["BoutEnd"]       = end + firstFrame
       item["TailAngle_Raw"] = angle[start:end+1,0].tolist()
-      
+
+      if hyperparameters["saveAllDataEvenIfNotInBouts"] and not hyperparameters['noBoutsDetection']:
+        for frameData in trackingFlatten[start:end+1]:
+          frameData[-1] = i
+
       if hyperparameters["eyeTracking"]:
         item["leftEyeX"]      = trackingEyesAllAnimals[animalId, start:end+1, 0].tolist()
         item["leftEyeY"]      = trackingEyesAllAnimals[animalId, start:end+1, 1].tolist()
@@ -360,7 +367,38 @@ def extractParameters(trackingData, wellNumber, hyperparameters, videoPath, well
         item["TailX_VideoReferential"] = tailX[start:end+1].tolist()
         item["TailY_VideoReferential"] = tailY[start:end+1].tolist()
       data.append(item)
-  
+
+    if hyperparameters["saveAllDataEvenIfNotInBouts"] or hyperparameters["storeH5"]:
+      trackingFlattenPandas = pd.DataFrame(trackingFlatten, columns=trackingFlattenColumnsNames)
+    if hyperparameters["saveAllDataEvenIfNotInBouts"]:
+      fname = os.path.join(hyperparameters["outputFolder"], hyperparameters["videoName"], f'allData_{hyperparameters["videoName"]}_wellNumber{wellNumber}_animal{animalId}.csv')
+      with open(fname, 'w+', newline='') as f:
+        if hyperparameters["videoFPS"]:
+          f.write(f'videoFPS: {hyperparameters["videoFPS"]}\n')
+        if hyperparameters["videoPixelSize"]:
+          f.write(f'videoPixelSize: {hyperparameters["videoPixelSize"]}\n')
+        trackingFlattenPandas.convert_dtypes().to_csv(f)
+    if hyperparameters['storeH5']:
+      with h5py.File(hyperparameters['H5filename'], 'a') as results:
+        group = results.create_group(f"dataForWell{wellNumber}/dataForAnimal{animalId}/dataPerFrame")
+        datasets = {'HeadPos': (('HeadPosX', 'HeadPosY'), ('X', 'Y')),
+                    'Heading': 'Heading',
+                    'TailAngle': 'tailAngle',
+                    'TailPosX': (tuple(f'TailPosX{pos}' for pos in range(1, nbPoints)), tuple(f'Pos{pos}' for pos in range(1, nbPoints))),
+                    'TailPosY': (tuple(f'TailPosY{pos}' for pos in range(1, nbPoints)), tuple(f'Pos{pos}' for pos in range(1, nbPoints))),
+                    'TailLength': 'TailLength'}
+        for name, columns in datasets.items():
+          if isinstance(columns, str):  # 1d array
+            group.create_dataset(name, data=trackingFlattenPandas[columns].values)
+          else:
+            assert isinstance(columns, tuple)  # 2d array
+            pandasNames, h5Names = columns
+            headPosData = np.empty(len(trackingFlattenPandas.index), dtype=list(zip(h5Names, (trackingFlattenPandas[name].dtype for name in pandasNames))))
+            for h5Name, pandasName in zip(h5Names, pandasNames):
+              headPosData[h5Name] = trackingFlattenPandas[pandasName].values
+            headPos = group.create_dataset(name, data=headPosData)
+            headPos.attrs['columns'] = headPosData.dtype.names
+
   print("Parameters extracted for well",wellNumber)
   if hyperparameters["popUpAlgoFollow"]:
     import zebrazoom.code.popUpAlgoFollow as popUpAlgoFollow
